package sun.security.util;

import java.security.SecureRandom;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.InvalidKeyException;
import java.security.InvalidAlgorithmParameterException;
import sun.misc.SharedSecrets;
import java.security.PublicKey;
import sun.security.rsa.RSAUtil;
import java.security.spec.ECParameterSpec;
import java.util.Locale;
import java.security.spec.AlgorithmParameterSpec;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.AlgorithmParameters;
import java.security.ProviderException;
import java.security.Signature;

public class SignatureUtil
{
    public SignatureUtil() {
    }
    
    private static String checkName(String algorithm) throws ProviderException {
        if (algorithm.indexOf(".") == -1) {
            return algorithm;
        }
        try {
            return Signature.getInstance(algorithm).getAlgorithm();
        }
        catch (Exception cause) {
            throw new ProviderException("Error mapping algorithm name", cause);
        }
    }
    
    private static AlgorithmParameters createAlgorithmParameters(String checkName, byte[] params) throws ProviderException {
        try {
            checkName = checkName(checkName);
            AlgorithmParameters instance = AlgorithmParameters.getInstance(checkName);
            instance.init(params);
            return instance;
        }
        catch (NoSuchAlgorithmException | IOException cause) {
            throw new ProviderException(cause);
        }
    }
    
    public static AlgorithmParameterSpec getParamSpec(String upperCase, AlgorithmParameters algorithmParameters) throws ProviderException {
        upperCase = checkName(upperCase).toUpperCase(Locale.ENGLISH);
        AlgorithmParameterSpec algorithmParameterSpec = null;
        if (algorithmParameters != null) {
            if (algorithmParameters.getAlgorithm().indexOf(".") != -1) {
                try {
                    algorithmParameters = createAlgorithmParameters(upperCase, algorithmParameters.getEncoded());
                }
                catch (IOException cause) {
                    throw new ProviderException(cause);
                }
            }
            if (upperCase.indexOf("RSA") != -1) {
                algorithmParameterSpec = RSAUtil.getParamSpec(algorithmParameters);
            }
            else if (upperCase.indexOf("ECDSA") != -1) {
                try {
                    algorithmParameterSpec = algorithmParameters.getParameterSpec(ECParameterSpec.class);
                }
                catch (Exception cause2) {
                    throw new ProviderException("Error handling EC parameters", cause2);
                }
            }
            else {
                throw new ProviderException("Unrecognized algorithm for signature parameters " + upperCase);
            }
        }
        return algorithmParameterSpec;
    }
    
    public static AlgorithmParameterSpec getParamSpec(String upperCase, byte[] array) throws ProviderException {
        upperCase = checkName(upperCase).toUpperCase(Locale.ENGLISH);
        AlgorithmParameterSpec algorithmParameterSpec = null;
        if (array != null) {
            if (upperCase.indexOf("RSA") != -1) {
                algorithmParameterSpec = RSAUtil.getParamSpec(createAlgorithmParameters(upperCase, array));
            }
            else if (upperCase.indexOf("ECDSA") != -1) {
                try {
                    algorithmParameterSpec = ECUtil.getECParameterSpec(Signature.getInstance(upperCase).getProvider(), array);
                }
                catch (Exception cause) {
                    throw new ProviderException("Error handling EC parameters", cause);
                }
                if (algorithmParameterSpec == null) {
                    throw new ProviderException("Error handling EC parameters");
                }
            }
            else {
                throw new ProviderException("Unrecognized algorithm for signature parameters " + upperCase);
            }
        }
        return algorithmParameterSpec;
    }
    
    public static void initVerifyWithParam(Signature signature, PublicKey publicKey, AlgorithmParameterSpec algorithmParameterSpec) throws ProviderException, InvalidAlgorithmParameterException, InvalidKeyException {
        SharedSecrets.getJavaSecuritySignatureAccess().initVerify(signature, publicKey, algorithmParameterSpec);
    }
    
    public static void initVerifyWithParam(Signature signature, Certificate certificate, AlgorithmParameterSpec algorithmParameterSpec) throws ProviderException, InvalidAlgorithmParameterException, InvalidKeyException {
        SharedSecrets.getJavaSecuritySignatureAccess().initVerify(signature, certificate, algorithmParameterSpec);
    }
    
    public static void initSignWithParam(Signature signature, PrivateKey privateKey, AlgorithmParameterSpec algorithmParameterSpec, SecureRandom secureRandom) throws ProviderException, InvalidAlgorithmParameterException, InvalidKeyException {
        SharedSecrets.getJavaSecuritySignatureAccess().initSign(signature, privateKey, algorithmParameterSpec, secureRandom);
    }
}
